#!/bin/bash
################################################################################
# Function: control the cases execute
# Platform: amd64/arm64/mips64el/loongarch64
# Author: Songcc
# Update: 2023/12/21 更新filter文件属主和属组检查逻辑。
# Update: 2023/12/20 更新DESKTOP_LIST检查排除目录，Version字段~字符。
# Update: 2023/12/19 更新Description检查逻辑，新增dos文件转换unix。
# Update: 2023/11/22 新增packageinfo.csv和reportsummary.csv报告，修改reportdetail.csv报告名称和标题。
# Update: 2023/11/08 在终端增加结果输出，优化测试报告。
# Date: 2023/10/26
# Company: Kylin Software Co.,Ltd.
################################################################################

current_path=$(cd $(dirname $0); pwd)
userpath="/home/$(whoami)"

function check_directory(){

    if [ ! -d "${userpath}/temp/" ]; then
        mkdir ${userpath}/temp/
    fi
    if [ ! -d "${userpath}/report/" ]; then
        mkdir ${userpath}/report/
    fi
}

# 全局只读变量定义
function common_info_definition(){
    readonly TMPPACKAGE="${userpath}/temp/tmppackage"    # 解包目录
    readonly TMPPACKAGELIST="${userpath}/temp/tmppackage.list"    # 包列表临时保存文件
    readonly CASESPOOL="${current_path}/casespool.csv"    # 测试用例文件路径
    readonly CASESMODIFY="${current_path}/casesmodify.csv"    # 测试用例修改建议文件路径
    readonly CURRENT_TIME=`date -d "1970-01-01 UTC $(date "+%s") seconds" "+%Y%m%d%H%M%S"` # date "+%s"获取当前秒数, 根据秒数转换成文件格式时间
    # readonly REPORT="./report/report_${CURRENT_TIME}.csv"
    readonly REPORT="${userpath}/report/reportdetail.csv"
    readonly REPORTSUMMARY="${userpath}/report/reportsummary.csv"
    readonly PACKAGEINFO="${userpath}/report/packageinfo.csv"
    readonly PASS_content="通过,,"
    readonly NA_content="不涉及,,"
    readonly FAIL_content="不通过"
    readonly WARN_content="警告"
}


# 错误日志
function log_err(){
    content="[ERROR] $(date '+%Y-%m-%d %H:%M:%S') $@"
    # echo "$content" >> $LOG_FILE && echo -e "\033[31m" ${content} "\033[0m"
    echo -e "\033[31m" ${content} "\033[0m"
}


# 写报告标题
function wirte_report_title(){
    local report_title=`head -n1 "$CASESPOOL"`
    echo "${report_title}" > "$REPORT"
    echo "测试总结果,成功数量,失败数量,警告数量,不涉及数量" > "$REPORTSUMMARY"
    echo "软件包名称,版本号,架构,开发者" > "$PACKAGEINFO"
}


# 将dos格式文件转换为unix格式
# 用法: file_dos_to_unix <文件路径>
function file_dos_to_unix(){
    dos_file_path="$1"
    if file "${dos_file_path}" |grep "with CRLF line terminators" &>/dev/null; then
        dos_file_path_format=`echo "${dos_file_path}" |sed "s@^$TMPPACKAGE@.@"`
        mv "${dos_file_path}" "${dos_file_path}.bak"
        tr -d '\r' < "${dos_file_path}.bak" > "${dos_file_path}"
    fi
}


# 解包以及对应文件列表环境准备
# 用法: environment_information_prepare <包名或包路径>
function environment_information_prepare(){
    DEB_PATH=`realpath "$1"`
    DEB_NAME=`echo "${DEB_PATH}" |awk -F / '{print $NF}'`
    dpkg-deb -R "${DEB_PATH}" "$TMPPACKAGE" &>/dev/null
    if [ $? -ne 0 ]; then
        log_err "${DEB_NAME}包损坏，解压缩失败，请上传新的驱动包"
        environment_exit
    fi
    find "$TMPPACKAGE" -type f -o -type l > "$TMPPACKAGELIST"
    CONTROL_FILE="$TMPPACKAGE/DEBIAN/control"
    # dos文件格式转换为unix
    file_dos_to_unix "${CONTROL_FILE}"
    PACKAGE=`awk -F : '{if($1=="Package") print $2}' "$CONTROL_FILE" |sed 's/ //g'`
    VERSION=`awk -F : '{if($1=="Version") print $2}' "$CONTROL_FILE" |sed 's/ //g'`
    ARCHITECTURE=`awk -F : '{if($1=="Architecture") print $2}' "$CONTROL_FILE" |sed 's/ //g'`
    MAINTAINER=`awk -F : '{if($1=="Maintainer") print $2}' "$CONTROL_FILE" |sed 's/ //g'`
    # DESCRIPTION=`awk -F : '{if($1=="Description") print $2}' "$CONTROL_FILE" |sed 's/ //g'`
    DESCRIPTION=`grep -c "^Description:" "$CONTROL_FILE"`
    DESKTOP_LIST_NUM=`grep -c "^$TMPPACKAGE/usr/share/applications/[^/]*[.]desktop$" "$TMPPACKAGELIST"`
    DESKTOP_LIST=`grep "^$TMPPACKAGE/usr/share/applications/[^/]*[.]desktop$" "$TMPPACKAGELIST"`
    RULES_LIST=`grep "[.]rules$" "$TMPPACKAGELIST"`
    PPD_LIST=`grep -E "[.]ppd$|[.]PPD$|[.]ppd.gz$|[.]PPD.gz$|[.]PPD.GZ$" "$TMPPACKAGELIST"`
    SANE_LIST=`awk -F / '$NF~"^libsane-(.+[.]so).*"{print $0}' "$TMPPACKAGELIST"`
    KO_LIST=`grep "[.]ko$" "$TMPPACKAGELIST"`
    SO_LIST=`awk -F / '$NF~".*[.]so.*"{print $0}' "$TMPPACKAGELIST"`
}


# 捕获CTRL+C，当脚本被ctrl+c的形式终止时清理环境
trap 'onCtrlC' INT
function onCtrlC(){
    rm -rf "${TMPPACKAGE}" "${LOG_FILE}" "${REPORT}" "${TMPPACKAGELIST}" "$REPORTSUMMARY" "$PACKAGEINFO"
    echo -e "\033[31m Ctrl+C is captured \033[0m"
    exit 1
}


# 完成清理环境
function environment_clean(){
    rm -rf "${TMPPACKAGE}" "${TMPPACKAGELIST}"
}


# 异常退出
function environment_exit(){
    rm -rf "${TMPPACKAGE}" "${LOG_FILE}" "${REPORT}" "${TMPPACKAGELIST}" "$REPORTSUMMARY" "$PACKAGEINFO"
    exit 1
}


# 从现有的casespool和casesmodify文件中获取用例和修改建议
# 用法: get_cases_modify_content "case01"  # 需要传入用例编号的参数，感觉用例编号提取对应用例和修改建议
function get_cases_modify_content(){
    casesname=`grep ^$1 "$CASESPOOL" |sed 's/\,\,\,\,//'`
    # casesmodify=`grep ^$1 "$CASESMODIFY" |awk -F , '{print $3}'`
    casesmodify=`grep ^$1 "$CASESMODIFY" |awk -F , '{print $3","$4}'`
}


# 模块: 包命名规范检查
# 包含用例: case01, case02
function check_debname_isCompliant(){
    # case01 命名规范检查: 检查驱动包名是否与{Package字段值}_{Version字段值}_{Architecture字段值}.deb完全相同
    get_cases_modify_content "common-standards-01"
    if [ "${DEB_NAME}" != "$PACKAGE"_"$VERSION"_"$ARCHITECTURE".deb ]; then
        echo "$casesname","${FAIL_content}","${DEB_NAME}包命名不符合规范","$casesmodify${PACKAGE}_${VERSION}_${ARCHITECTURE}.deb" >> "$REPORT"
    else
        echo "$casesname","${PASS_content}" >> "$REPORT"
    fi

    # case02 包命名长度检查: 检查驱动包名长度是否小于等于255
    get_cases_modify_content "common-standards-02"
    local deb_new_name="${PACKAGE}"_"${VERSION}"_"${ARCHITECTURE}".deb
    local deb_len=`echo ${#deb_new_name}`
    if [ "${deb_len}" -gt 255 ]; then
        echo "$casesname","${FAIL_content}","${deb_new_name}包名长度大于255","$casesmodify" >> "$REPORT"
    else
        echo "$casesname","${PASS_content}" >> "$REPORT"
    fi
}


# 模块: 包control文件规范检查
# 包含用例: case03, case04, case05, case06, case07, case08, case09, case10, case11, case12
function check_control_file_isCompliant(){
    # case03 control字段值冒号空格检查: 检查control文件各字段名冒号后面是否有一个空格
    get_cases_modify_content "common-standards-03"
    # local param_count=`cat "$CONTROL_FILE" |grep -Ev "^$|^[[:blank:]]" |awk -F : '{print $2}' |grep -cv "^ "`
    local param_count=`awk -F : '{if($0!~"^$" && $0!~"^[[:blank:]]" && $2!~"^ ") print}' "$CONTROL_FILE" |wc -l`
    if [ "${param_count}" -ne 0 ]; then
        # 获取不符合规范内容的行数
        local param_contents=`awk -F : '{if($0!~"^$" && $0!~"^[[:blank:]]") print $2}' "$CONTROL_FILE" |grep -nv "^ " |awk -F : '{print $1}'`
        local param_content_value_args
        for param_content in ${param_contents}; do
            # sed打印对应不符合规范的行数的内容存入数组
            local param_content_value=`grep -Ev "^$|^[[:blank:]]" "$CONTROL_FILE" |sed -n ${param_content}p |awk -F : '{print $1}'`
            param_content_value_args[${#param_content_value_args[*]}]="${param_content_value}:"
        done
        echo "$casesname","${FAIL_content}","control文件中${param_content_value_args[*]}字段冒号后没有空格","$casesmodify" >> "$REPORT"
    else
        echo "$casesname","${PASS_content}" >> "$REPORT"
    fi

    # case04 control文件字段首字母检查: 检查control文件各字段首字母是否大写
    get_cases_modify_content "common-standards-04"
    # local param_first=`cat "$CONTROL_FILE" |grep -Ev "^$|^[[:blank:]]" | awk -F : '{print $1}' | grep -cv -e ^[A-Z]`
    local param_first=`awk -F : '{if($0!~"^$" && $0!~"^[[:blank:]]") print $1}' "$CONTROL_FILE" |grep -cv -e ^[A-Z]`
    if [ "${param_first}" -ne 0 ]; then
        local param_first_content=`awk -F : '{if($0!~"^$" && $0!~"^[[:blank:]]") print $1}' "$CONTROL_FILE" |grep -v -e ^[A-Z]`
        echo "$casesname","${FAIL_content}","control文件中${param_first_content}字段首字母没有大写","$casesmodify" >> "$REPORT"
    else
        echo "$casesname","${PASS_content}" >> "$REPORT"
    fi

    # case05 control文件空字段检查: 检查control文件各字段值是否不为空
    get_cases_modify_content "common-standards-05"
    # local space_num=`awk -F : '{if($0!~"^$" && $0!~"^[[:blank:]]") print $2}' "$CONTROL_FILE" |grep -c '^[[:blank:]]*$'`
    local space_num=`grep -v "^Description:" "${CONTROL_FILE}" |awk -F : '{if($0!~"^$" && $0!~"^[[:blank:]]") print $2}' |grep -c '^[[:blank:]]*$'`
    if [ "${space_num}" -ne 0 ]; then
        # local space_num_lines=`awk -F : '{if($0!~"^$" && $0!~"^[[:blank:]]") print $2}' "$CONTROL_FILE" |grep -n '^[[:blank:]]*$' |awk -F : '{print $1}'`
        local space_num_lines=`grep -v "^Description:" "${CONTROL_FILE}" |awk -F : '{if($0!~"^$" && $0!~"^[[:blank:]]") print $2}' |grep -n '^[[:blank:]]*$' |awk -F : '{print $1}'`
        for space_num_line in ${space_num_lines}; do
            local space_num_line_content=`grep -v "^Description:" "${CONTROL_FILE}" |awk -F : '{if($0!~"^$" && $0!~"^[[:blank:]]") print $0}' |sed -n ${space_num_line}p`
            local space_content_args
            space_content_args[${#space_content_args[*]}]="${space_num_line_content}"
        done
        echo "$casesname","${FAIL_content}","control文件中${space_content_args[*]}字段值为空","$casesmodify" >> "$REPORT"
    else
        echo "$casesname","${PASS_content}" >> "$REPORT"
    fi

    # case06 control文件必填字段检查: 检查control文件中是否包含Package、Version、Architecture、Maintainer、Description字段
    get_cases_modify_content "common-standards-06"
    local param_isexist_args
    if [ "x$PACKAGE" == 'x' ]; then
        param_isexist_args[${#param_isexist_args[*]}]="Package"
    fi
    if [ "x$VERSION" == 'x' ]; then
        param_isexist_args[${#param_isexist_args[*]}]="Version"
    fi
    if [ "x$ARCHITECTURE" == 'x' ]; then
        param_isexist_args[${#param_isexist_args[*]}]="Architecture"
    fi
    if [ "x$MAINTAINER" == 'x' ]; then
        param_isexist_args[${#param_isexist_args[*]}]="Maintainer"
    fi
    # if [ "x$DESCRIPTION" == 'x' ]; then
    if [ "$DESCRIPTION" -eq 0 ]; then
        param_isexist_args[${#param_isexist_args[*]}]="Description"
    fi
    if [ ${#param_isexist_args[*]} -ne 0 ]; then
        echo "$casesname","${FAIL_content}","control文件中缺少${param_isexist_args[*]}字段","$casesmodify" >> "$REPORT"
    else
        echo "$casesname","${PASS_content}" >> "$REPORT"
    fi

    local section_lists="admin, cli-mono, comm, database, debug, devel, doc, editors, education, electronics, embedded, fonts, games, gnome, gnu-r, gnustep, graphics, hamradio, haskell, httpd, interpreters, introspection, java, javascript, kde, kernel, libdevel, libs, lisp, localization, mail, math, metapackages, misc, net, news, ocaml, oldlibs, otherosfs, perl, php, python, ruby, rust, science, shells, sound, tasks, tex, text, utils, vcs, video, web, x11, xfce, zope,"
    local priority_lists="required, important, standard, optional, extra,"
    local section=`awk -F : '{if($1=="Section") print $2}' "$CONTROL_FILE" |sed 's/ //g'`
    local priority=`awk -F : '{if($1=="Priority") print $2}' "$CONTROL_FILE" |sed 's/ //g'`
    local installed_size=`awk -F : '{if($1=="Installed-Size") print $2}' "$CONTROL_FILE" |sed 's/ //g'`
    local size=`awk -F : '{if($1=="Size") print $2}' "${CONTROL_FILE}" |sed 's/ //g'`

    # case07 control文件Package字段值规范性检查: 检查control文件中Package字段值是否不包含非法字符、是否以字母开头
    get_cases_modify_content "common-standards-07"
    local package_pass=`echo "$PACKAGE" |grep -o '[0-9a-z]\|-\|\.\|+' | awk '{{printf"%s",$0}}'`
    local package_first=`echo "$PACKAGE" |grep -cv ^[a-z]`
    if [ "$PACKAGE" != "${package_pass}" -o "${package_first}" -ne 0 ]; then
        echo "$casesname","${FAIL_content}","control文件中Package: ${PACKAGE}不符合规范","$casesmodify" >> "$REPORT"
    else
        echo "$casesname","${PASS_content}" >> "$REPORT"
    fi

    # case08 control文件Version字段值规范性检查: 检查control文件中Version字段值是否不包含非法字符、是否以数字开头
    get_cases_modify_content "common-standards-08"
    local version_pass=`echo "$VERSION" | grep -o '[0-9]\|-\|\.\|[a-zA-Z]\|+\|~' | awk '{{printf"%s",$0}}'`
    local version_first=`echo "$VERSION" | grep -cv ^[0-9]`
    if [ "$VERSION" != "${version_pass}" -o "${version_first}" -ne 0 ]; then
        echo "$casesname","${FAIL_content}","control文件中Version: ${VERSION}不符合规范","$casesmodify" >> "$REPORT"
    else
        echo "$casesname","${PASS_content}" >> "$REPORT"
    fi
  
    # case09 control文件Architecture字段值规范性检查: 检查control文件中Architecture字段值是否为指定架构平台对应的值或all
    get_cases_modify_content "common-standards-09"
    if [ "$ARCHITECTURE" != "amd64" ] && [ "$ARCHITECTURE" != "arm64" ] && [ "$ARCHITECTURE" != "mips64el" ] && [ "$ARCHITECTURE" != "loongarch64" ] && [ "$ARCHITECTURE" != "all" ]; then
        echo "$casesname","${FAIL_content}","control文件中Architecture: ${ARCHITECTURE}不符合规范","$casesmodify" >> "$REPORT"
    else
        echo "$casesname","${PASS_content}" >> "$REPORT"
    fi

    # case10 control文件Section字段值规范性检查: 如果control文件中存在Section字段，则其字段值必须是debian要求的固定list中的值
    get_cases_modify_content "common-standards-10"
    if [ "x$section" != x ]; then
        local section_num=`echo $section_lists |grep -co "$section,"`
        if [ "${section_num}" -eq 0 ]; then
            echo "$casesname","${FAIL_content}","control文件中Section: ${section}不符合规范","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi
    else
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi

    # case11 control文件Priority字段值规范性检查: 如果control文件中存在Priority字段，则其字段值必须是debian要求的固定list中的值
    get_cases_modify_content "common-standards-11"
    if [ "x$priority" != x ]; then
        local priority_num=`echo $priority_lists |grep -co "$priority,"`
        if [ "${priority_num}" -eq 0 ]; then
            echo "$casesname","${FAIL_content}","control文件中: ${priority}不符合规范","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi
    else
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi

    # case12 control文件Size和Installed-Size字段值规范性检查: 如果control文件中存在Size或Installed-Size字段，则其字段值必须是纯数字不能带单位
    get_cases_modify_content "common-standards-12"
    # if [ "x${installed_size}" != x ]; then
    if [ "x${installed_size}" != x -o "x$size" != x ]; then
        local size_content_err_args
        if [ "x${installed_size}" != x ]; then
            local installed_size_pass=`echo "${installed_size}" | grep -o '[0-9]' | awk '{{printf"%s",$0}}'`
   	        if [ "${installed_size}" != "${installed_size_pass}" ]; then
            #     echo "$casesname","${FAIL_content}","control文件中Installed-Size: ${installed_size}不符合规范","$casesmodify" >> "$REPORT"
            # else
            #     echo "$casesname","${PASS_content}" >> "$REPORT"
                size_content_err_args[${#size_content_err_args[*]}]="Installed_Size: ${installed_size}"
            fi
        fi
        if [ "x$size" != x ]; then
            local size_pass=`echo "$size" | grep -o '[0-9]' | awk '{{printf"%s",$0}}'`
   	        if [ "$size" != "${size_pass}" ]; then
                size_content_err_args[${#size_content_err_args[*]}]="Size: $size"
            fi
        fi
        if [ ${#size_content_err_args[*]} -ne 0 ]; then
            echo "$casesname","${FAIL_content}","control文件中${size_content_err_args[*]}不符合规范","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi
    else
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi
    # else
    #     echo "$casesname","${NA_content}" >> "$REPORT"
    # fi
}


# 模块: 包脚本文件内容规范检查
# 功能: 根据目录集和命令集遍历传入的文件
# 用法: check_dir_permissions_isCompliant "<文件路径>"
function check_dir_permissions_isCompliant(){
    local system_dir="/bin /usr/bin /lib /usr/lib /lib32 /usr/lib32 /lib64 /usr/lib64 /libx32 /usr/libx32 /sbin /usr/sbin /data /tmp /boot /cdrom /dev /etc /home /media /mnt /opt /run /srv /usr /var /proc /sys /lost+found /root"
    local system_cmd="chmod chown rm rmdir mv"
    local script_path="$1"
    for check_system_cmd in ${system_cmd}; do
        for check_system_dir in ${system_dir};do
            if grep -v "^#" "$script_path" |grep -P "${check_system_cmd}.*( ${check_system_dir}(/?)$| ${check_system_dir}(/?)( +))" &>/dev/null; then
                # 输出匹配到的不合规内容的"文件名:行数:内容"
                local script_content_err=`grep -nHv "^#" "$script_path" |grep -P "${check_system_cmd}.*( ${check_system_dir}(/?)$| ${check_system_dir}(/?)( +))" |sed "s@^$TMPPACKAGE@.@"`
                modify_system_dir_args[${#modify_system_dir_args[*]}]="${script_content_err}"
            fi
        done
    done
    modify_system_dir_args=($(awk -vRS=' ' '!a[$1]++' <<< ${modify_system_dir_args[@]}))
}


# 模块: 包脚本文件内容规范检查
# 功能: 查找传入文件中包含.sh后缀的脚本文件并检查该脚本中的内容是否包含不合规的命令和目录
# 用法: check_invoke_script_isCompliant "<文件路径>"
function check_invoke_script_isCompliant(){
    local script_path="$1"
    local invoke_script_name=`awk '$NF~".*[.]sh" {print $NF}' "${script_path}" |awk -F / '{print $NF}' |uniq`
        if [ "x${invoke_script_name}" != x ]; then
            for invoke_script in ${invoke_script_name}; do
                local invoke_script_path=`cat "$TMPPACKAGELIST" |grep "${invoke_script}"`
                if [ -f "${invoke_script_path}" ]; then
                    check_dir_permissions_isCompliant "${invoke_script_path}"
                fi
            done
        fi
}


# 模块: 包脚本文件内容规范检查
# 包含用例: case13
function check_script_content_isCompliant(){
    # case13 包安装卸载脚本内容检查: 如果包中存在preinst、prerm、postinst、postrm脚本则检查脚本中是否修改、替换、删除系统的重要目录及权限。
    modify_system_dir_args=()
    get_cases_modify_content "common-standards-13"
    if [ -f "$TMPPACKAGE/DEBIAN/preinst" -o -f "$TMPPACKAGE/DEBIAN/prerm" -o -f "$TMPPACKAGE/DEBIAN/postinst" -o -f "$TMPPACKAGE/DEBIAN/postrm" ]; then
        if [ -f "$TMPPACKAGE/DEBIAN/preinst" ]; then
            check_dir_permissions_isCompliant "$TMPPACKAGE/DEBIAN/preinst"
            check_invoke_script_isCompliant "$TMPPACKAGE/DEBIAN/preinst"
        fi
        if [ -f "$TMPPACKAGE/DEBIAN/prerm" ]; then
            check_dir_permissions_isCompliant "$TMPPACKAGE/DEBIAN/prerm"
            check_invoke_script_isCompliant "$TMPPACKAGE/DEBIAN/prerm"
        fi
        if [ -f "$TMPPACKAGE/DEBIAN/postinst" ]; then
            check_dir_permissions_isCompliant "$TMPPACKAGE/DEBIAN/postinst"
            check_invoke_script_isCompliant "$TMPPACKAGE/DEBIAN/postinst"
        fi
        if [ -f "$TMPPACKAGE/DEBIAN/postrm" ]; then
            check_dir_permissions_isCompliant "$TMPPACKAGE/DEBIAN/postrm"
            check_invoke_script_isCompliant "$TMPPACKAGE/DEBIAN/postrm"
        fi
        if [ ${#modify_system_dir_args[*]} -ne 0 ]; then
            echo "$casesname","${WARN_content}","不符合规范的脚本:行数:内容为${modify_system_dir_args[*]}","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi
    else
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi
}


# 模块: 包desktop文件规范检查
# 包含用例: case14, case15, case16, case17, case18, case19, case20
function check_desktop_file_isCompliant(){
    if [ "${DESKTOP_LIST_NUM}" -ge 1 ]; then
        # local desktop_path_err_args
        local app_name_null_args
        local app_name_zh_null_args
        local app_icon_null_args
        local app_exec_null_args
        local app_type_null_args
        for desktop_path in ${DESKTOP_LIST}; do
            file_dos_to_unix "${desktop_path}"
            # sed双引号传入变量，变量中含有/与sed分隔符冲突，将sed分隔符换成@
            local desktop_path_format=`echo "${desktop_path}" |sed "s@^$TMPPACKAGE@.@"`
            local desktop_name=`echo "${desktop_path}" |awk -F / '{print $NF}'`
            local app_name=`awk -F = '/^Name=/{print $2}' "${desktop_path}"`
            local app_name_zh=`awk -F = '/^Name\[zh_CN\]=/{print }' "${desktop_path}"`
            local app_icon=`awk -F = '/^Icon=/{print $2}' "${desktop_path}"`
            local app_exec=`awk -F = '/^Exec=/{print $2}' "${desktop_path}"`
            local app_type=`awk -F = '/^Type=/{print $2}' "${desktop_path}"`

            # case15 desktop文件存储路径规范检查: 检查包中的.desktop文件是否存储在系统路径/usr/share/applications目录下
            # if [ "x${desktop_path_pass}" == x ]; then
            # if [ ! -f "$TMPPACKAGE/usr/share/applications/${desktop_name}" ]; then
            #     desktop_path_err_args[${#desktop_path_err_args[*]}]="${desktop_path_format}"
            # fi

            # case16 desktop文件Name必填字段检查: 检查Name字段是否存在且值不为空
            if [ "x${app_name}" == x ]; then
                app_name_null_args[${#app_name_null_args[*]}]="${desktop_path_format}"
            fi

            # case17 desktop文件Name[zh_CN]必填字段检查: 检查Name[zh_CN]字段是否存在且值不为空
            if [ "x${app_name_zh}" == x ]; then
                app_name_zh_null_args[${#app_name_zh_null_args[*]}]="${desktop_path_format}"
            fi

            # case18 desktop文件Icon必填字段检查: 检查Icon字段是否存在且值不为空以及驱动包中是否存在此icon文件
            if [ "x${app_icon}" == x ]; then
                app_icon_null_args[${#app_icon_null_args[*]}]="${desktop_path_format}"
            else
                if ! grep "${app_icon}" "$TMPPACKAGELIST" &>/dev/null; then
                    app_icon_null_args[${#app_icon_null_args[*]}]="${desktop_path_format}"
                fi
            fi

            # case19 desktop文件Exec必填字段检查: 检查Exec字段是否存在且值不为空
            if [ "x${app_exec}" == x ]; then
                app_exec_null_args[${#app_exec_null_args[*]}]="${desktop_path_format}"
            fi

            # case20 desktop文件Type必填字段检查: 检查Type字段是否存在且值为Application、Link、Direcory其中一种
            if [ "${app_type}" != Application ] && [ "${app_type}" != Link ] && [ "${app_type}" != Direcory ]; then
                app_type_null_args[${#app_type_null_args[*]}]="${desktop_path_format}"
            fi
        done

        # case14 desktop文件命名检查: 如果包中只存在一个.desktop文件则检查Desktop文件名是否为{Package字段值}.desktop
        get_cases_modify_content "common-standards-14"
        if [ "${DESKTOP_LIST_NUM}" -eq 1 ]; then
            local desktop_name=`echo "${DESKTOP_LIST}" |awk -F / '{print $NF}'`
            if [ "${desktop_name}" == "$PACKAGE.desktop" ]; then
                echo "$casesname","${PASS_content}" >> "$REPORT"
            else
                echo "$casesname","${WARN_content}","${desktop_name}文件命名不符合规范","${casesmodify}${PACKAGE}.desktop" >> "$REPORT"
            fi
        else
            echo "$casesname","${NA_content}" >> "$REPORT"
        fi

        # case15 desktop文件存储路径规范检查: 检查包中的.desktop文件是否存储在/usr/share/applications/目录下
        get_cases_modify_content "common-standards-15"
        # if [ ${#desktop_path_err_args[*]} -ne 0 ]; then
        #     echo "$casesname","FAIL","${desktop_path_err_args[*]}","$casesmodify" >> "$REPORT"
        # else
        echo "$casesname","${PASS_content}" >> "$REPORT"
        # fi

        # case16 desktop文件Name必填字段检查: 检查Name字段是否存在且值不为空
        get_cases_modify_content "common-standards-16"
        if [ ${#app_name_null_args[*]} -ne 0 ]; then
            echo "$casesname","${FAIL_content}","desktop文件${app_name_null_args[*]}中缺少Name字段","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi

        # case17 desktop文件Name[zh_CN]必填字段检查: 检查Name[zh_CN]字段是否存在且值不为空
        get_cases_modify_content "common-standards-17"
        if [ ${#app_name_zh_null_args[*]} -ne 0 ]; then
            echo "$casesname","${FAIL_content}","desktop文件${app_name_zh_null_args[*]}中缺少Name[zh_CN]字段","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi

        # case18 desktop文件Icon必填字段检查: 检查Icon字段是否存在且值不为空以及驱动包中是否存在此icon文件
        get_cases_modify_content "common-standards-18"
        if [ ${#app_icon_null_args[*]} -ne 0 ]; then
            echo "$casesname","${FAIL_content}","desktop文件${app_icon_null_args[*]}中缺少Icon字段","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi

        # case19 desktop文件Exec必填字段检查: 检查Exec字段是否存在且值不为空
        get_cases_modify_content "common-standards-19"
        if [ ${#app_exec_null_args[*]} -ne 0 ]; then
            echo "$casesname","${FAIL_content}","desktop文件${app_exec_null_args[*]}中缺少Exec字段","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi

        # case20 desktop文件Type必填字段检查: 检查Type字段是否存在且值为Application、Link、Direcory其中一种
        get_cases_modify_content "common-standards-20"
        if [ ${#app_type_null_args[*]} -ne 0 ]; then
            echo "$casesname","${FAIL_content}","desktop文件${app_type_null_args[*]}中缺少Type字段","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi

    else
        # log_info "$DEB_NAME驱动包中没有desktop文件"
        get_cases_modify_content "common-standards-14"
        echo "$casesname","${NA_content}" >> "$REPORT"
        get_cases_modify_content "common-standards-15"
        echo "$casesname","${NA_content}" >> "$REPORT"
        get_cases_modify_content "common-standards-16"
        echo "$casesname","${NA_content}" >> "$REPORT"
        get_cases_modify_content "common-standards-17"
        echo "$casesname","${NA_content}" >> "$REPORT"
        get_cases_modify_content "common-standards-18"
        echo "$casesname","${NA_content}" >> "$REPORT"
        get_cases_modify_content "common-standards-19"
        echo "$casesname","${NA_content}" >> "$REPORT"
        get_cases_modify_content "common-standards-20"
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi
}


# 模块: 包图标文件规范检查
# 包含用例: case21, case22
function check_icon_file_isCompliant(){
    if [ "${DESKTOP_LIST_NUM}" -ge 1 ]; then
        local icon_path_err_args
        for desktop_path in ${DESKTOP_LIST}; do
            local icon_name=`grep ^Icon= "${desktop_path}" |awk -F = '{print $2}' |awk -F / '{print $NF}'`
            # 获取到的icon字段是以svg结尾
            if echo "${icon_name}" |grep "[.]svg$" &>/dev/null; then
                if [ ! -f "$TMPPACKAGE/usr/share/icons/hicolor/scalable/apps/${icon_name}" ]; then
                    icon_path_err_args[${#icon_path_err_args[*]}]="${icon_name}"
                fi
            # 获取到的icon字段是以png结尾
            elif echo "${icon_name}" |grep "[.]png$" &>/dev/null; then
                if [ ! -f "$TMPPACKAGE/usr/share/icons/hicolor/16x16/apps/${icon_name}" \
                -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/24x24/apps/${icon_name}" \
                -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/32x32/apps/${icon_name}" \
                -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/48x48/apps/${icon_name}" \
                -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/64x64/apps/${icon_name}" \
                -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/96x96/apps/${icon_name}" \
                -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/128x128/apps/${icon_name}" \
                -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/256x256/apps/${icon_name}" \
                -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/512x512/apps/${icon_name}" ]; then
                    icon_path_err_args[${#icon_path_err_args[*]}]="${icon_name}"
                fi
            # 获取到的icon字段没有后缀
            else
                local icon_name_svg="${icon_name}.svg"
                local icon_name_png="${icon_name}.png"
                if [ ! -f "$TMPPACKAGE/usr/share/icons/hicolor/scalable/apps/${icon_name_svg}" ]; then
                    if [ ! -f "$TMPPACKAGE/usr/share/icons/hicolor/16x16/apps/${icon_name_png}" \
                    -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/24x24/apps/${icon_name_png}" \
                    -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/32x32/apps/${icon_name_png}" \
                    -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/48x48/apps/${icon_name_png}" \
                    -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/64x64/apps/${icon_name_png}" \
                    -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/96x96/apps/${icon_name_png}" \
                    -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/128x128/apps/${icon_name_png}" \
                    -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/256x256/apps/${icon_name_png}" \
                    -o ! -f "$TMPPACKAGE/usr/share/icons/hicolor/512x512/apps/${icon_name_png}" ]; then
                        icon_path_err_args[${#icon_path_err_args[*]}]="${icon_name}"
                    fi
                fi
            fi
        done

        # case21 图标文件命名检查: 检查图标文件命名是否为{Package字段值}.svg或{Package字段值}.png
        get_cases_modify_content "common-standards-21"
        if [ "${DESKTOP_LIST_NUM}" -eq 1 ]; then
            local icon_name=`cat "${DESKTOP_LIST}" |grep ^Icon= |awk -F = '{print $2}'`
            local icon_name_pass=`echo "${icon_name}" |awk -F / '{print $NF}' |sed -r 's/\.png$|\.svg$//'`
            if [ "${icon_name_pass}" == "$PACKAGE" ]; then
                echo "$casesname","${PASS_content}" >> "$REPORT"
            else
                echo "$casesname","${WARN_content}","${icon_name}图标文件命名不符合规范","${casesmodify}${PACKAGE}.svg或${PACKAGE}.png" >> "$REPORT"
            fi
        else
            echo "$casesname","${NA_content}" >> "$REPORT"
        fi

        # case22 图标文件存储路径规范检查: 检查包中的.png或.svg图标文件是否存储在系统指定路径下
        get_cases_modify_content "common-standards-22"
        if [ ${#icon_path_err_args[*]} -ne 0 ]; then
            echo "$casesname","${WARN_content}","${icon_path_err_args[*]}图标文件没有存储在系统指定目录下","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi
    else
        get_cases_modify_content "common-standards-21"
        echo "$casesname","${NA_content}" >> "$REPORT"
        get_cases_modify_content "common-standards-22"
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi
}


# 模块: 包rules文件规范检查
# 包含用例: case23, case24, case25, case26
function check_rules_file_isCompliant(){
    if [ "x${RULES_LIST}" != x ]; then
        local rules_first_err_args
        local rules_num_err_args
        local rules_path_err_args
        local rules_permissions_err_args

        # case23 rules文件命名开头数字检查: 检查rules文件名称是否是以数字-开头
        for rules_path in ${RULES_LIST}; do
            local rules_path_format=`echo "${rules_path}" |sed "s@^$TMPPACKAGE@.@"`
            local rules_name=`echo "${rules_path}" |awk -F / '{print $NF}'`
            # local rules_path_pass=`echo "${rules_path}" |grep "$TMPPACKAGE/etc/uedv/rules.d/.*[.]rules"`
            local rules_permissions=`stat -L -c %a "${rules_path}"`

            if echo "${rules_name}" |grep ^[0-9]*- &>/dev/null; then

                # case24 rules文件命名开头数字值大小检查: 检查rules文件名称开头的数字是否大于50
                local rules_num=`echo "${rules_name}" |grep -o ^[0-9]*`
                if [ "${rules_num}" -le 50 ]; then
                    rules_num_err_args[${#rules_num_err_args[*]}]="${rules_path_format}"
                fi
            else
                rules_first_err_args[${#rules_first_err_args[*]}]="${rules_path_format}"
            fi

            # case25 rules文件存储路径规范检查: 检查包中的.rules文件是否在系统路径/etc/udev/rules.d目录下
            if [ ! -f "$TMPPACKAGE/etc/udev/rules.d/${rules_name}" ]; then
            # if [ "x${rules_path_pass}" == x ]; then
                rules_path_err_args[${#rules_path_err_args[*]}]="${rules_path_format}"
            fi

            # case26 rules文件权限规范检查: 检查包中的.rules文件在系统中的权限是否等于644
            if [ "${rules_permissions}" -ne 644 ]; then
                rules_permissions_err_args[${#rules_permissions_err_args[*]}]="${rules_path_format}"
            fi
        done

        get_cases_modify_content "common-standards-23"
        if [ ${#rules_first_err_args[*]} -ne 0 ]; then
            echo "$casesname","${WARN_content}","${rules_first_err_args[*]}的rules文件名不是以数字开头","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi

        get_cases_modify_content "common-standards-24"
        if [ ${#rules_num_err_args[*]} -ne 0 ]; then
            echo "$casesname","${FAIL_content}","${rules_num_err_args[*]}的rules文件名开头的数字小于等于50 该rules文件不生效","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi

        get_cases_modify_content "common-standards-25"
        if [ ${#rules_path_err_args[*]} -ne 0 ]; then
            echo "$casesname","${WARN_content}","rules文件${rules_path_err_args[*]}没有存储在系统指定目录下","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi

        get_cases_modify_content "common-standards-26"
        if [ ${#rules_permissions_err_args[*]} -ne 0 ]; then
            echo "$casesname","${WARN_content}","rules文件${rules_permissions_err_args[*]}的权限不是644","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi

    else
        get_cases_modify_content "common-standards-23"
        echo "$casesname","${NA_content}" >> "$REPORT"
        get_cases_modify_content "common-standards-24"
        echo "$casesname","${NA_content}" >> "$REPORT"
        get_cases_modify_content "common-standards-25"
        echo "$casesname","${NA_content}" >> "$REPORT"
        get_cases_modify_content "common-standards-26"
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi
}


# 模块: 软件包文件存储路径规范检查
# 包含用例: case27, case28
function check_software_install_path_isCompliant(){
    # case27 检查应用安装文件存储路径: 检查应用安装文件是否存储在/opt/apps/{Package字段值}/目录下
    get_cases_modify_content "common-standards-27"
    if [ -d "$TMPPACKAGE/opt/apps/$PACKAGE" ]; then
        echo "$casesname","${PASS_content}" >> "$REPORT"
    else
        echo "$casesname","${WARN_content}","未找到软件安装目录","$casesmodify/opt/apps/$PACKAGE/" >> "$REPORT"
    fi

    # case28 检查动态库文件存储路径: 检查动态库文件是否存储在/opt/apps/{Package字段值}/lib/目录下
    get_cases_modify_content "common-standards-28"
    if [ "x${SO_LIST}" != x ]; then
        local so_path_err_args
        for so_path in ${SO_LIST}; do
            local so_path_format=`echo "${so_path}" |sed "s@^$TMPPACKAGE@.@"`
            local so_name=`echo "${so_path}" |awk -F / '{print $NF}'`
        # local rules_name=`echo "${rules_path}" |awk -F / '{print $NF}'`
            # local so_path_pass=`echo "${so_path}" |grep "$TMPPACKAGE/opt/apps/$PACKAGE/lib/"`
            if [ ! -f "$TMPPACKAGE/opt/apps/$PACKAGE/lib/${so_name}" ]; then
            # if [ "x${so_path_pass}" == x ]; then
                so_path_err_args[${#so_path_err_args[*]}]="${so_path_format}"
            fi
        done

        if [ ${#so_path_err_args[*]} -ne 0 ]; then
            echo "$casesname","${WARN_content}","软件包动态库文件${so_path_err_args[*]}没有存储在系统指定目录下","$casesmodify/opt/apps/$PACKAGE/lib/" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi
    else
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi
}


# 模块: 驱动包命名规范检查
# 包含用例: case29
function check_all_binary_isExist(){
    # case29 驱动包架构命名规范检查: 如果包为all架构则检查包中是否存在与架构环境相关的二进制可执行程序
    get_cases_modify_content "common-standards-29"
    if [ "$ARCHITECTURE" == all ]; then
        read x86_binary arm_binary loongarch_binary mips_binary <<< $(echo 0 0 0 0)
        IFS_bak=$IFS
        IFS=$'\n'
        for all_files in `cat "$TMPPACKAGELIST"`; do
            if file "${all_files}" |grep "ELF 64-bit LSB" |grep "x86-64" &>/dev/null; then
                x86_binary=1
            elif file "${all_files}" |grep "ELF 64-bit LSB" |grep "ARM" &>/dev/null; then
                arm_binary=1
            elif file "${all_files}" |grep "ELF 64-bit LSB" |grep -E "unknown|loongarch64|LoongArch" &>/dev/null; then
                loongarch_binary=1
            elif file "${all_files}" |grep "ELF 64-bit LSB" |grep "MIPS" &>/dev/null; then
                mips_binary=1
            fi
        done
        IFS=$IFS_bak
        if [ "${x86_binary}" -eq 1 -a "${arm_binary}" -eq 1 -a "${loongarch_binary}" -eq 1 -a "${mips_binary}" -eq 1 ] || [ "${x86_binary}" -eq 0 -a "${arm_binary}" -eq 0 -a "${loongarch_binary}" -eq 0 -a "${mips_binary}" -eq 0 ]; then
            echo "$casesname","${PASS_content}" >> "$REPORT"
        else
            echo "$casesname","${WARN_content}","${DEB_NAME}包中包含依赖架构的可执行文件","$casesmodify" >> "$REPORT"
        fi
    else
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi
}


# 模块: 驱动包文件存储路径规范检查
# 包含用例: case30, case31, case32, case33, case34
function check_driver_path_isCompliant(){
    if [ "x${PPD_LIST}" != x ]; then
        IFS_bak=$IFS
        IFS=$'\n'
        filter_list_args=()
        local ppd_path_err_args
        local filter_path_err_args
        for ppd_path in ${PPD_LIST}; do
            local ppd_name=`echo "${ppd_path}" |awk -F / '{print $NF}'`
            local ppd_path_format=`echo "${ppd_path}" |sed "s@^$TMPPACKAGE@.@"`
            local ppd_path_pass=`echo "${ppd_path}" |grep "^$TMPPACKAGE/usr/share/cups/model/.*${ppd_name}$"`
            if [ "x${ppd_path_pass}" == x ]; then
                ppd_path_err_args[${#ppd_path_err_args[*]}]="${ppd_path_format}"
            fi

            if echo "${ppd_path}" |grep -i gz$ &>/dev/null; then
                local filter=`zgrep "^*cupsFilter:" "${ppd_path}" |awk -F \" '{print $2}' |awk '{print $3}'`
                local filter2=`zgrep "^*cupsFilter2:" "${ppd_path}" |awk -F \" '{print $2}' |awk '{print $1}' |awk -F - '{print $2}'`
            else
                local filter=`grep "^*cupsFilter:" "${ppd_path}" |awk -F \" '{print $2}' |awk '{print $3}'`
                local filter2=`grep "^*cupsFilter2:" "${ppd_path}" |awk -F \" '{print $2}' |awk '{print $1}' |awk -F - '{print $2}'`
            fi
            filter_list_args[${#filter_list_args[*]}]="${filter}"
            filter_list_args[${#filter_list_args[*]}]="${filter2}"
        done

        # filter数组去重
        filter_list_args=($(awk -vRS=' ' '!a[$1]++' <<< ${filter_list_args[@]}))
        if [ ${#filter_list_args[*]} -ne 0 ]; then
            for filter_name in ${filter_list_args[@]}; do
                local filter_path_pass=`grep "^$TMPPACKAGE/usr/lib/cups/filter/${filter_name}" "$TMPPACKAGELIST"`
                # 软链接失效的情况会导致if -f判断不到文件所以用grep匹配
                if [ "x${filter_path_pass}" == x ]; then
                # if [ ! -f "$TMPPACKAGE/usr/lib/cups/filter/${filter_name}" ]; then
                    filter_path_err_args[${#filter_path_err_args[*]}]="${filter_name}"
                fi
            done
        fi
        IFS=$IFS_bak

        # case30 打印机PPD文件存储路径检查: 检查驱动包中的PPD文件是否存储在/usr/share/cups/model/目录或其子目录下
        get_cases_modify_content "common-standards-30"
        if [ ${#ppd_path_err_args[*]} -ne 0 ]; then
            echo "$casesname","${FAIL_content}","PPD文件${ppd_path_err_args[*]}没有存储在系统指定目录下","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi

        # case31 打印机Filter文件存储路径检查: 检查驱动包中的Filter文件是否存储在/usr/lib/cups/filter/目录下
        get_cases_modify_content "common-standards-31"
        if [ ${#filter_list_args[*]} -ne 0 ]; then
            if [ ${#filter_path_err_args[*]} -ne 0 ]; then
                echo "$casesname","${FAIL_content}","Filter文件${filter_path_err_args[*]}没有存储在系统指定目录下","$casesmodify" >> "$REPORT"
            else
                echo "$casesname","${PASS_content}" >> "$REPORT"
            fi
        else
            echo "$casesname","${NA_content}" >> "$REPORT"
        fi
    else
        get_cases_modify_content "common-standards-30"
        echo "$casesname","${NA_content}" >> "$REPORT"

        get_cases_modify_content "common-standards-31"
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi

    if [ "x${SANE_LIST}" != x ]; then
        local sane_path_err_args
        local dll_path_err_args
        if [ "$ARCHITECTURE" == "amd64" ]; then
            local libsane_dir="/usr/lib/x86_64-linux-gnu/sane"
            local libsane_dir_suggest="/usr/lib/x86_64-linux-gnu/sane/"
        elif [ "$ARCHITECTURE" == "arm64" ]; then
            local libsane_dir="/usr/lib/aarch64-linux-gnu/sane"
            local libsane_dir_suggest="/usr/lib/aarch64-linux-gnu/sane/"
        elif [ "$ARCHITECTURE" == "loongarch64" ]; then
            local libsane_dir="/usr/lib/loongarch64-linux-gnu/sane"
            local libsane_dir_suggest="/usr/lib/loongarch64-linux-gnu/sane/"
        elif [ "$ARCHITECTURE" == "mips64el" ]; then
            local libsane_dir="/usr/lib/mips64el-linux-gnu.*/sane"
            local libsane_dir_suggest="V10: /usr/lib/mips64el-linux-gnu/sane/; V10SP1: /usr/lib/mips64el-linux-gnuabi64/sane/"
        else
            local libsane_dir="/usr/lib/*-linux-gnu/sane/"
            local libsane_dir_suggest="/usr/lib/<架构>-linux-gnu/sane/"
        fi

        for sane_path in ${SANE_LIST}; do
            local sane_path_format=`echo "${sane_path}" |sed "s@^$TMPPACKAGE@.@"`
            local sane_name=`echo "${sane_path}" |awk -F / '{print $NF}'`
            local sane_path_pass=`echo "${sane_path}" |grep "^$TMPPACKAGE${libsane_dir}/${sane_name}$"`
            # 软链接失效的情况会导致if -f判断不到文件所以用grep匹配
            if [ "x${sane_path_pass}" == x ]; then
            # if [ ! -f "$TMPPACKAGE/${libsane_dir}/${sane_name}" ]; then
                sane_path_err_args[${#sane_path_err_args[*]}]="${sane_path_format}"
            fi

            local dll_name=`echo "${sane_name}" |sed -r 's/^libsane-|\.so.*//g'`
            # local dll_path_pass=`cat "$TMPPACKAGELIST" |grep "$TMPPACKAGE/etc/sane[.]d/dll[.]d/${dll_name}"`
            if ! grep -r "${dll_name}" "$TMPPACKAGE/etc/sane.d/dll.d/" &>/dev/null; then
                dll_path_err_args[${#dll_path_err_args[*]}]="${dll_name}"
            fi
        done
        dll_path_err_args=($(awk -vRS=' ' '!a[$1]++' <<< ${dll_path_err_args[@]}))

        # case32 扫描仪libsane文件存储路径检查: 检查驱动包中以libsane开头的扫描.so动态库文件是否存储在此架构平台对应的路径下
        get_cases_modify_content "common-standards-32"
        if [ ${#sane_path_err_args[*]} -ne 0 ]; then
            echo "$casesname","${WARN_content}","libsane文件${sane_path_err_args[*]}没有存储在系统指定目录下","$casesmodify${libsane_dir_suggest}" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi

        # case33: 扫描仪dll后端文件存储路径检查: 检查驱动包中扫描dll后端文件是否存储在/etc/sane.d/dll.d/目录下
        get_cases_modify_content "common-standards-33"
        if [ ${#dll_path_err_args[*]} -ne 0 ]; then
            echo "$casesname","${WARN_content}","dll文件${dll_path_err_args[*]}没有存储在系统指定目录下","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi
    else
        get_cases_modify_content "common-standards-32"
        echo "$casesname","${NA_content}" >> "$REPORT"
        get_cases_modify_content "common-standards-33"
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi

    if [ "x${KO_LIST}" != x ]; then
        local ko_path_err_args
        # local kernel_version=`uname -r`

        for ko_path in ${KO_LIST}; do
            local ko_path_format=`echo "${ko_path}" |sed "s@^$TMPPACKAGE@.@"`
            local ko_name=`echo "${ko_path}" |awk -F / '{print $NF}'`
            local ko_path_pass=`echo "${ko_path}" |grep "^$TMPPACKAGE/lib/modules/.*/kernel/.*${ko_name}$"`
            if [ "x${ko_path_pass}" == x ]; then
                ko_path_err_args[${#ko_path_err_args[*]}]="${ko_path_format}"
            fi
        done

        # case34 内核.ko文件存储路径检查: 检查驱动包中的内核.ko文件是否存储在/lib/modules/$(uname -r)/kernel目录或其子目录下
        get_cases_modify_content "common-standards-34"
        if [ ${#ko_path_err_args[*]} -ne 0 ]; then
            echo "$casesname","${WARN_content}","ko文件${ko_path_err_args[*]}没有存储在系统指定目录下","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi
    else
        get_cases_modify_content "common-standards-34"
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi
}


# 模块: 驱动包文件权限规范检查
# 包含用例: case35, case36, case37, case38, case39
function check_driver_permissions_isCompliant(){
    if [ "x${PPD_LIST}" != x ]; then
        IFS_bak=$IFS
        IFS=$'\n'
        local ppd_permissions_err_args
        local filter_permissions_err_args
        for ppd_path in ${PPD_LIST}; do
            local ppd_path_format=`echo "${ppd_path}" |sed "s@^$TMPPACKAGE@.@"`
            local ppd_permissions=`stat -L -c %a ${ppd_path}`
            if [ "${ppd_permissions}" -ne 644 ]; then
                ppd_permissions_err_args[${#ppd_permissions_err_args[*]}]="${ppd_path_format}"
            fi
        done

        if [ ${#filter_list_args[*]} -ne 0 ]; then
            for filter_name in ${filter_list_args[@]}; do
                local filter_path_num=`grep -c "/${filter_name}$" "$TMPPACKAGELIST"`
                if [ "${filter_path_num}" -ge 1 ]; then
                    for filter_path in `grep "/${filter_name}$" "$TMPPACKAGELIST"`; do
                        # 软链接为绝对路径的情况未安装驱动时链接是失效的，if -f找不到对应文件，判断软链接使用readlink获取链接的路径
                        if [ ! -f "${filter_path}" ]; then
                            if [ -L "${filter_path}" ]; then
                                local filter_path_new=`reaklink -f "${filter_path}"`
                                local filter_path="$TMPPACKAGE${filter_path_new}"
                            fi
                        fi
                        local filter_path_format=`echo "${filter_path}" |sed "s@^$TMPPACKAGE@.@"`
                        local filter_permissions=`stat -L -c %a "${filter_path}"`
                        local filter_userid=`dpkg -c  "${DEB_PATH}" |grep "${filter_path_format}" |awk '{print $2}' |awk -F / '{print $1}'`
                        local filter_groupid=`dpkg -c  "${DEB_PATH}" |grep "${filter_path_format}" |awk '{print $2}' |awk -F / '{print $2}'`
                        if [ "${filter_permissions}" -ne 755 -o "${filter_userid}" != "root" -o "${filter_groupid}" != "root" ]; then
                            filter_permissions_err_args[${#filter_permissions_err_args[*]}]="${filter_path_format}"
                        fi
                    done
                fi
            done
        fi
        IFS=$IFS_bak

        # case35 打印机PPD文件权限检查: 检查驱动包中的PPD文件权限是否等于644
        get_cases_modify_content "common-standards-35"
        if [ ${#ppd_permissions_err_args[*]} -ne 0 ]; then
            echo "$casesname","${FAIL_content}","PPD文件${ppd_permissions_err_args[*]}的权限不是644","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi

        # case36 打印机Filter文件权限检查: 检查驱动包中的Filter文件权限是否等于755且属组和属主均为root
        get_cases_modify_content "common-standards-36"
        if [ ${#filter_list_args[*]} -ne 0 ]; then
            if [ ${#filter_permissions_err_args[*]} -ne 0 ]; then
                echo "$casesname","${FAIL_content}","Filter文件${filter_permissions_err_args[*]}的权限/属主/属组不是755/root/root 该打印驱动无法正常打印","$casesmodify" >> "$REPORT"
            else
                echo "$casesname","${PASS_content}" >> "$REPORT"
            fi
        else
            echo "$casesname","${NA_content}" >> "$REPORT"
        fi
    else
        get_cases_modify_content "common-standards-35"
        echo "$casesname","${NA_content}" >> "$REPORT"
        get_cases_modify_content "common-standards-36"
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi

    if [ "x${SANE_LIST}" != x ]; then
        local sane_permissions_err_args
        local dll_permissions_err_args
        for sane_path in ${SANE_LIST}; do
            # 软链接为绝对路径的情况未安装驱动时链接是失效的，if -f找不到对应文件，判断软链接使用readlink获取链接的路径
            if [ ! -f "${sane_path}" ]; then
                if [ -L "${sane_path}" ]; then
                    local sane_path_new=`readlink -f "${sane_path}"`
                    local sane_path="$TMPPACKAGE${sane_path_new}"
                # else
                #     continue
                fi
            fi
            local sane_path_format=`echo "${sane_path}" |sed "s@^$TMPPACKAGE@.@"`
            local sane_permissions=`stat -L -c %a ${sane_path}`
            if [ "${sane_permissions}" -ne 644 ]; then
                sane_permissions_err_args[${#sane_permissions_err_args[*]}]="${sane_path_format}"
            fi

            local dll_name=`echo "${sane_path}" |awk -F / '{print $NF}' |sed -r 's/^libsane-|\.so.*//g'`
            if grep -r "${dll_name}" "$TMPPACKAGE/etc/sane.d/dll.d/" &>/dev/null; then
                local dll_path=`grep -r "${dll_name}" "$TMPPACKAGE/etc/sane.d/dll.d/" |awk -F : '{print $1}' |uniq`
                local dll_path_format=`echo "${dll_path}" |sed "s@^$TMPPACKAGE@.@"`
                local dll_permissions=`stat -L -c %a ${dll_path}`
                if [ "${dll_permissions}" -ne 644 ]; then
                    dll_permissions_err_args[${#dll_permissions_err_args[*]}]="${dll_path_format}"
                fi
            fi
        done

        dll_permissions_err_args=($(awk -vRS=' ' '!a[$1]++' <<< ${dll_permissions_err_args[@]}))

        # case37 扫描仪libsane文件权限检查: 检查驱动包中以libsane开头的扫描.so动态库文件权限是否等于644
        get_cases_modify_content "common-standards-37"
        if [ ${#sane_permissions_err_args[*]} -ne 0 ]; then
            echo "$casesname","${WARN_content}","libsane文件${sane_permissions_err_args[*]}的权限不是644","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi

        # case38 扫描仪dll后端文件权限检查: 检查驱动包中扫描dll后端文件的权限是否等于644
        get_cases_modify_content "common-standards-38"
        if [ ${#dll_permissions_err_args[*]} -ne 0 ]; then
            echo "$casesname","${WARN_content}","dll文件${dll_permissions_err_args[*]}的权限不是644","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi
    else
        get_cases_modify_content "common-standards-37"
        echo "$casesname","${NA_content}" >> "$REPORT"
        get_cases_modify_content "common-standards-38"
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi


    if [ "x${KO_LIST}" != x ]; then
        local ko_permissions_err_args
        for ko_path in ${KO_LIST}; do
            local ko_path_format=`echo "${ko_path}" |sed "s@^$TMPPACKAGE@.@"`
            local ko_permissions=`stat -c %a ${ko_path}`
            if [ "${ko_permissions}" -ne 644 ]; then
                ko_permissions_err_args[${#ko_permissions_err_args[*]}]="${ko_path_format}"
            fi
        done

        # case39 内核.ko文件权限检查: 检查驱动包中的内核.ko文件权限是否等于644
        get_cases_modify_content "common-standards-39"
        if [ ${#ko_permissions_err_args[*]} -ne 0 ]; then
            echo "$casesname","${WARN_content}","ko文件${ko_permissions_err_args[*]}的权限不是644","$casesmodify" >> "$REPORT"
        else
            echo "$casesname","${PASS_content}" >> "$REPORT"
        fi
    else
        get_cases_modify_content "common-standards-39"
        echo "$casesname","${NA_content}" >> "$REPORT"
    fi
}


# 模块: 测试结果终端输出
function print_result_inTerminal(){
    local report_absolute_path=`realpath "$REPORT"`
    local reportsummary_absolute_path=`realpath "$REPORTSUMMARY"`
    local packageinfo_absolute_path=`realpath "$PACKAGEINFO"`
    local casesall_num=`tail -n +2 "$REPORT" |awk -F , '{print $4}' |wc -l`
    local casespass_num=`tail -n +2 "$REPORT" |awk -F , '{print $4}' |grep -c "^通过"`
    local casesfail_num=`tail -n +2 "$REPORT" |awk -F , '{print $4}' |grep -c "^不通过"`
    local caseswarn_num=`tail -n +2 "$REPORT" |awk -F , '{print $4}' |grep -c "^警告"`
    local casesna_num=`tail -n +2 "$REPORT" |awk -F , '{print $4}' |grep -c "^不涉及"`
    if [ "${casesfail_num}" -ne 0 ]; then
        local report_result="不通过"
    else
        local report_result="通过"
    fi

    echo "${report_result},${casespass_num},${casesfail_num},${caseswarn_num},${casesna_num}" >> "$REPORTSUMMARY"
    echo "${PACKAGE}","${VERSION}","${ARCHITECTURE}",\"${MAINTAINER}\" >> "$PACKAGEINFO"

    echo "
测试详细结果路径为: ${report_absolute_path}
测试结果概述路径为: ${reportsummary_absolute_path}
包信息的保存路径为: ${packageinfo_absolute_path}
规范性检查结果: ${report_result}
用例总数${casesall_num}条: 通过${casespass_num}条, 不通过${casesfail_num}条, 警告${caseswarn_num}条, 不涉及${casesna_num}条。
    "
}


# 模块: 软件包测试用例执行
function software_cases_exec(){
    check_debname_isCompliant
    check_control_file_isCompliant
    check_script_content_isCompliant
    check_desktop_file_isCompliant
    check_icon_file_isCompliant
    check_rules_file_isCompliant
    check_software_install_path_isCompliant
    # 将NA结果替换成FAIL
    if grep "common-standards-14" "$REPORT" |grep "${NA_content}" &>/dev/null; then
        sed -i -r "/^common-standards-14|^common-standards-15|^common-standards-16|^common-standards-17|^common-standards-18|^common-standards-19|^common-standards-20/s@${NA_content}@${FAIL_content},usr/share/applications/目录下未找到.desktop文件,,请编写desktop文件用来打开应用程序@" "$REPORT"
    fi
}


# 模块: 外设包测试用例执行
function peripheral_cases_exec(){
    check_debname_isCompliant
    check_control_file_isCompliant
    check_script_content_isCompliant
    check_desktop_file_isCompliant
    check_icon_file_isCompliant
    check_rules_file_isCompliant
    check_all_binary_isExist
    check_driver_path_isCompliant
    check_driver_permissions_isCompliant
}
