#!/bin/bash
################################################################################
# Function: DEB package specification check base on kylin desktop os
# Platform: amd64/arm64/mips64el/loongarch64
# Author: Songcc
# Date: 2023/10/26
# Company: Kylin Software Co.,Ltd.
################################################################################

current_path=$(cd $(dirname $0); pwd)

source ${current_path}/casecontrol.sh

# 使用帮助
function Usage_help(){
   echo "
此工具用于DEB包规范检查
   "
    exit 1
}

# 1.使用方法说明：
# 执行文件 参数1 参数2
# 1) 执行文件: run.sh
# 2) 参数1: 待测软件包类型，参数不区分大小写，包括: 
# software  软件包;
# peripheral  外设包
# 3) 参数2: DEB包路径

# 2.使用举例：
# ./run.sh 待测软件包类型 软件包名称，如: 
# ./run.sh software  /home/kylin/fcitx_2.2.1-1_arm64.deb
# ./run.sh --help  显示帮助文档

# 判断传参是否符合规范
function check_paramnum_isCompliant()
{
    local param_num="$#"
    local check_type="$1"
    if [ "${param_num}" -ne 2 ]; then
        Usage_help
    fi
    case "${check_type}" in
        software|SOFTWARE)
            strategy_ID="software"
            ;;
        peripheral|PERIPHERAL)
            strategy_ID="peripheral"
            ;;
        *)
            Usage_help
            ;;
    esac
    local check_package=`realpath "$2"`
    if ! file "${check_package}" |grep "Debian binary package" &>/dev/null; then
        Usage_help
    fi
}


# 主程序
function main(){
    check_paramnum_isCompliant $@
    common_info_definition
    check_directory
    wirte_report_title
    environment_information_prepare $2
    if [ "${strategy_ID}" == "software" ]; then
        software_cases_exec
    elif [ "${strategy_ID}" == "peripheral" ]; then
        peripheral_cases_exec
    fi
    environment_clean
    print_result_inTerminal
}


main $@
